
DROP TABLE aus_user_ancestors;

DROP TABLE aus_session;

DROP TABLE aus_user_log;
DROP TABLE aus_user_flags;
DROP TABLE aus_user_membership;
DROP TABLE aus_user;

DROP TABLE aus_password_crypt;
DROP TABLE aus_flag;
