
DROP TRIGGER aus_user_on_insert ON aus_user;
DROP TRIGGER aus_user_on_delete ON aus_user;
DROP TRIGGER aus_user_on_update ON aus_user;
DROP TRIGGER aus_user_membership_on_insert ON aus_user_membership;
DROP TRIGGER aus_user_membership_on_delete ON aus_user_membership;
